class Word:
    def __init__(self,
                 word,
                 start_time,
                 end_time,
                 confidence):
        """
        :param word: any word in transcription
        :param start_time: start time of word in milliseconds
        :param end_time: end time of word in milliseconds
        :param confidence: confidence score of word
        """
        self.word = word
        self.start_time = start_time
        self.end_time = end_time
        self.confidence = confidence

    def dump_json(self):
        json_desc = '{\n'
        json_desc += '\t\"word\": \"' + self.word + '\",\n'
        json_desc += '\t\"start_time\": ' + str(self.start_time) + ',\n'
        json_desc += '\t\"end_time\": ' + str(self.end_time) + ',\n'
        json_desc += '\t\"confidence\": ' + str(self.confidence) + '\n'
        json_desc += '}'
        return json_desc

    @classmethod
    def parse_json(cls, json):
        """Creates instance from JSON description of word"""
        return cls(
            json["word"],
            json["startTime"],
            json["endTime"],
            json["confidence"]
        )



from enum import Enum

class PaymentType(Enum):
    PAYMENT = 1
    SUBSCRIPTION = 2

    def to_string(self):
        return self.name.lower()

    @classmethod
    def from_string(cls, payment_type):
        return cls[payment_type.upper()]



from enum import Enum

class JobStatus(Enum):
	WAITING = -1
	PREPARING = 0
	PROCESSING = 1
	TRANSCRIBING = 2
	READY = 3
	FAILED = 4

	def to_string(self):
		return self.name.lower()

	@classmethod
	def from_string(cls, status):
		return cls[status.upper()]



from .subscription_status import SubscriptionStatus

class Subscription:
    def __init__(self,
                 created_on,
                 expires_on,
                 status):
        """
        :param created_on: the date and time subscription starts
        :param expires_on: the date and time subscription ends
        :param status: status of subscription. Can be one of: ACTIVE, EXPIRED, SUSPENDED, CANCELLED, TERMINATED
        """

        self.created_on = created_on
        self.expires_on = expires_on
        self.status = status

    def dump_json(self):
        json_desc = '{\n'
        json_desc += '\t\"created_on\": \"' + self.created_on + '\",\n'
        json_desc += '\t\"expires_on\": \"' + self.expires_on + '\",\n'
        json_desc += '\t\"status\": \"' + self.status.to_string() + '\"\n'
        json_desc += '}'
        return json_desc

    @classmethod
    def parse_json(cls, json):
        """Creates instance from json description of subscription"""
        if(json is None):
            return None

        return cls(
            json["createdOn"],
            json["expiresOn"],
            SubscriptionStatus.from_string(json["status"])
        )




import requests
from urllib.parse import urljoin
from urllib.parse import quote
from requests.exceptions import HTTPError
from .classes import Job
from .classes import Transcription
from .classes import Account
from .classes import Payment

class VoicenClient:
	#Current version voicen RESTful API
	api_version = "v1"

	#Base URL of API
	api_base_url = "https://api.voicen.com/speechtotext/{}/".format(api_version)
#	api_base_url = "http://127.0.0.1:8080/speechtotext/{}/".format(api_version)

	def __init__(self, voicen_access_token):
		if not voicen_access_token:
			raise ValueError("You must provide access token to use API.")

		self.default_headers = {
			"Authorization": "Bearer {}".format(voicen_access_token)
		}

	def create_job_with_media_url(self, media_url, language):
		"""
			Creates transcription job using media URL. After creating job downloads audio from media URL and transcribes it.

			:param media_url: web url of audio or video. Voicen currently supports transcribing of youtube.
			:param language: language of speech downloaded from media url
			:returns: new created job
			:raises: HTTPError
		"""
		#media url is important and must be included
		if not media_url:
			raise ValueError("You must provide media URL to create job.")

		#language of speech is also important and must be included
		if not language:
			raise ValueError("You must provide language of speech on media URL.")

		#copy of default HTTP headers to use additional headers
		headers = self.default_headers.copy()
		headers["Content-Type"] = "application/x-www-form-urlencoded"
		headers["Accept"] = "application/json"
		headers["Cache-Control"] = "no-cache"
		
		payload = "lang={}&link={}".format(quote(language), quote(media_url))

		response = requests.request("POST", urljoin(self.api_base_url, "jobs/url"), data=payload, headers=headers)

		try:
			response.raise_for_status()
			return Job.parse_json(response.json())
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def create_job_with_local_file(self, local_file, language):
		"""
			Creates transcription job using local audio or video file. 
			Uploads local media file to the server.
			After creating job processes uploaded media file.

			:param local_file: local audio or video file to upload and transcribe
			:param language: language of speech of local audio or video file
			:returns: new created job
			:raises: HTTPError
		"""

		#media file is important and must be included
		if not local_file:
			raise ValueError("You must provide local video ro audio file to create job.")

		#language of speech is also important and must be included
		if not language:
			raise ValueError("You must provide language of speech of local video or audio file.")

		#copy of default HTTP headers to use additional headers
		headers = self.default_headers.copy()
		#headers["Content-Type"] = "multipart/form-data"
		headers["Accept"] = "application/json"
		headers["Cache-Control"] = "no-cache"
		#headers["content-Type"] = "multipart/form-data"
		
		
		payload = {"lang": language}

		try:
			files={'file':open(local_file, "rb")}
#			payload['file'] = local_file #open(local_file, "rb")
		except IOError as err:
			raise

		response = requests.request("POST", urljoin(self.api_base_url, "jobs/file"), data=payload, headers=headers, files = files)

		try:
			response.raise_for_status()
			return Job.parse_json(response.json())
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def get_job(self, job_id):
		"""
		Retrieves job details by id.

		:param: job_id: ID of job want to get details
		:returns: job
		:raises: HTTPError
		"""
		
		if not job_id:
			raise ValueError("You must provide valid job ID value.")

		#copy of default HTTP headers to use additional headers
		headers = self.default_headers.copy()
		headers["Accept"] = "application/json"
		headers["Cache-Control"] = "no-cache"
		
		response = requests.request("GET", urljoin(self.api_base_url, "jobs/{}".format(job_id)), headers=headers)

		try:
			response.raise_for_status()
			return Job.parse_json(response.json())
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def get_jobs(self, start_after = 0, limit = 100):
		"""
		Retrieves job list realtive to start_after and limit parameters.

		:param: start_after: the job ID that you retrieve job list which their id greater than it.
		:param: limit: max size of job list
		:returns: array of job
		:raises: HTTPError
		"""

		if start_after is None:
			raise ValueError("You must provide valid start_after param.")

		if limit is None:
			raise ValueError("You must provide valid limit param.")

		#copy of default HTTP headers to use additional headers
		headers = self.default_headers.copy()
		headers["Accept"] = "application/json"
		headers["Cache-Control"] = "no-cache"

		response = requests.request("GET", urljoin(self.api_base_url, "jobs?start_after={}&limit={}".format(start_after, limit)), headers=headers)

		try:
			response.raise_for_status()
			return [Job.parse_json(job) for job in response.json()]
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def delete_job(self, job_id):
		"""
		Deletes job and it's all data by ID

		:param job_id: ID of job which will be deleted
		:raises: HTTPError
		:returns: None if job deleted successfully
		"""
		if not job_id:
			raise ValueError("You must provide valid job ID value.")

		headers = self.default_headers.copy()
		headers["Accept"] = "application/json"
		headers["Cache-Control"] = "no-cache"

		response = requests.request("DELETE", urljoin(self.api_base_url, "jobs/{}".format(job_id)), headers=headers)

		try:
			response.raise_for_status()
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

		return

	def get_transcription(self, job_id):
		"""
		Retrieves transcript by job ID

		:param: job_id: ID of job want to get transcript
		:returns:  job transcription
		:raises: HTTPError
		"""

		if not job_id:
			raise ValueError("You must provide valid job ID value.")
		headers = self.default_headers.copy()
		headers["Cache-Control"] = "no-cache"

		response = requests.request("GET", urljoin(self.api_base_url, "jobs/{}/transcript?type=json".format(job_id)), headers=headers)

		try:
			response.raise_for_status()
			return Transcription.parse_json(response.json())
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise


	def get_transcription_as_text(self, job_id):
		"""
		Retrieves transcript as text by job ID

		:param: job_id: ID of job want to get transcript as text
		:returns: job transcription as text
		:raises: HTTPError
		"""

		if not job_id:
			raise ValueError("You must provide valid job ID value.")

		headers = self.default_headers.copy()
		headers["Cache-Control"] = "no-cache"

		response = requests.request("GET", urljoin(self.api_base_url, "jobs/{}/transcript?type=txt".format(job_id)), headers=headers)

		try:
			response.raise_for_status()
			return response.text
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def get_transcription_as_srt(self, job_id):
		"""
		Retrieves transcription as srt by job ID

		:param: job_id: ID of job want to get transcription as srt
		:returns: job transcription as srt
		:raises: HTTPError
		"""

		if not job_id:
			raise ValueError("You must provide valid job ID value.")
		headers = self.default_headers.copy()
		headers["Cache-Control"] = "no-cache"

		response = requests.request("GET", urljoin(self.api_base_url, "jobs/{}/transcript?type=srt".format(job_id)), headers=headers)

		try:
			response.raise_for_status()
			return response.text
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def get_transcription_as_msword(self, job_id):
		"""
		Retrieves transcript as microsoft word by job ID

		:param: job_id: ID of job want to get transcript as text
		:returns: job transcription as microsoft word
		:raises: HTTPError
		"""

		if not job_id:
			raise ValueError("You must provide valid job ID value.")

		headers = self.default_headers.copy()
		headers["Cache-Control"] = "no-cache"

		response = requests.request("GET", urljoin(self.api_base_url, "jobs/{}/transcript?type=doc".format(job_id)), headers=headers)

		try:
			response.raise_for_status()
			return response.content
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def get_account(self):
		"""
		Retrieves account details.

		:returns: account
		:raises: HTTPError
		"""

		#copy of default HTTP headers to use additional headers
		headers = self.default_headers.copy()
		headers["Accept"] = "application/json"
		headers["Cache-Control"] = "no-cache"

		response = requests.request("GET", urljoin(self.api_base_url, "account"), headers=headers)

		try:
			response.raise_for_status()
			return Account.parse_json(response.json())
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise

	def get_payments(self):
		"""
		Retrieves account details.

		:returns: account
		:raises: HTTPError
		"""

		#copy of default HTTP headers to use additional headers
		headers = self.default_headers.copy()
		headers["Accept"] = "application/json"
		headers["Cache-Control"] = "no-cache"

		response = requests.request("GET", urljoin(self.api_base_url, "payments"), headers=headers)

		try:
			response.raise_for_status()
			return [Payment.parse_json(payment) for payment in response.json()]
		except HTTPError as err:
			if response.content:
				err.args = (err.args[0] + "; Server Response: \n{}".format(response.content.decode("utf-8")),)
			raise



from .job_status import JobStatus

class Job():
	def __init__(
					self,
					job_id,
					name,
					language,
					duration,
					status,
					created,
					completed):
			"""
			:param job_id: unique identfication of job
			:param name: name of job
			:param language: language of speech
			:param duration: duration of speech
			:param status: current status of job. Can be one of: WAITING, PREPARING, PROCESSING, TRANSCRIBING, READY, FAILED
			:param created: creation time of job
			:param completion time of job
			"""

			self.id = job_id
			self.name = name
			self.language = language
			self.duration = duration
			self.status = status
			self.created = created
			self.completed = completed

	def dump_json(self):
		"""Makes JSON description of job"""
		json_desc = '{\n'
		json_desc += '\t\"id\": ' + str(self.id) + ',\n'
		json_desc += '\t\"name\": \"' + self.name + '\",\n'
		json_desc += '\t\"lamguage\": \"' + self.language + '\",\n'
		if self.duration is not None:
			json_desc += '\t\"duration\": ' + str(self.duration) + ',\n'
		json_desc += '\t\"status\": \"' + self.status.to_string() + '\",\n'
		json_desc += '\t\"created\": \"' + self.created
		if self.completed is not None:
			json_desc += '\",\n\t\"completed\": \"' + self.completed + '\"\n'
		else:
			json_desc += '\"\n'
		json_desc += "}"
		return json_desc

	@classmethod
	def parse_json(cls, json):
		"""Creates instance from JSON"""
		return cls(
			json["id"],
			json["name"],
			json["language"],
			json.get("duration"),
			JobStatus.from_string(json["status"]),
			json["created"],
			json.get("completed")
		)



from .payment_type import PaymentType
from .subscription import Subscription

class Payment:
    def __init__(self,
                 transaction_id,
                 amount,
                 type,
                 purchased_minutes,
                 created,
                 subscription):
        """
        :param transaction_id: transaction ID of payment
        :param amount: payment amount
        :param payment_type: type of payment. Can be one of: PAYMENT, SUBSCRIPTION
        :param purchased_minutes: purchased transcribing time in minutes
        :param created: date and time of payment
        :param subscription: if payment type is SUBSCRIPTION this field will not none.
        """

        self.transaction_id = transaction_id
        self.amount = amount
        self.type = type
        self.purchased_minutes = purchased_minutes
        self.created = created
        self.subscription = subscription

    def dump_json(self):
        json_desc = '{\n'
        json_desc += '\t\"transaction_id\": \"' + self.transaction_id + '\",\n'
        json_desc += '\t\"amount\": ' + str(self.amount) + ',\n'
        json_desc += '\t\"type\": \"' + self.type.to_string() + '\",\n'
        json_desc += '\t\"purchased_minutes\": ' + str(self.purchased_minutes) + ',\n'
        json_desc += '\t\"created\": \"' + self.created + '\"'
        if self.subscription is not None:
            json_desc += ',\n\t\"subscription\": '
            subscr_json = self.subscription.dump_json()
            subscr_json_lines = subscr_json.split('\n')
            for line in subscr_json_lines:
                if line != subscr_json_lines[0] and line != subscr_json_lines[-1]:
                    json_desc += '\t\t'
                if line == subscr_json_lines[-1]:
                    json_desc += '\t'
                json_desc += line
                json_desc += '\n'
        else:
            json_desc += '\n'
        json_desc += '}'
        return json_desc

    @classmethod
    def parse_json(cls, json):
        return cls(
            json["transactionId"],
            json["amount"],
            PaymentType.from_string(json["type"]),
            json.get("purchasedMinutes"),
            json.get("created"),
            Subscription.parse_json(json.get("subscription"))
        )



class Account:
    def __init__(self,
                 email,
                 balance_seconds):
        """
        :param email: email of account
        :param balance_seconds: balance of account in seconds
        """
        self.email = email
        self.balance_seconds = balance_seconds

    def dump_json(self):
        json_desc = '{\n'
        json_desc += '\t\"email\": \"' + self.email + '\",\n'
        json_desc += '\t\"balance_seconds\": ' + str(self.balance_seconds) + '\n'
        json_desc += '}'
        return json_desc

    @classmethod
    def parse_json(cls, json):
        """Creates instance from JSON description of account"""
        return cls(
            json["email"],
            json["balanceInSeconds"]
        )



from .job import Job
from .job_status import JobStatus
from .account import Account
from .payment import Payment
from .payment_type import PaymentType
from .subscription import Subscription
from .subscription_status import SubscriptionStatus
from .transcription import Transcription
from .word import Word



from .word import Word

class Transcription:
    def __init__(self, text = "", words = []):
        """
        :param text: text of transcription
        :param words: detailed words of transcription
        """
        self.text = text
        self.words = words

    def dump_json(self):
        json_desc = '{\n'
        json_desc += '\t\"text\": \"' + self.text + '\",\n'
        json_desc += '\t\"words\": [\n'
        for word in self.words:
            word_json = word.dump_json()
            word_json_lines = word_json.split('\n')
            for line in word_json_lines:
                json_desc += '\t\t' + line
                if line != word_json_lines[-1]:
                    json_desc += '\n'
            if word != self.words[-1]:
                json_desc += ','
            json_desc += '\n'
        json_desc += '\t]\n'
        json_desc += '}'
        return json_desc

    @classmethod
    def parse_json(cls, json):
        """JSON description of transcription"""
        return cls(
            json["fulltext"],
            [Word.parse_json(word) for word in json["words"]]
        )



#!/usr/bin/env python

from glob import glob
from setuptools import setup, find_packages
from os import path
try:  # for pip >= 10
    from pip._internal.req import parse_requirements
    from pip._internal.download import PipSession
except ImportError:  # for pip <= 9.0.3
    from pip.req import parse_requirements
    from pip.download import PipSession
import re
import ast
import os

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('src/voicen/__init__.py', 'rb') as f:
    version = str(ast.literal_eval(_version_re.search(
        f.read().decode('utf-8')).group(1)))

this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, 'README.md')) as fh:
	long_description = fh.read()

parsed_requirements = parse_requirements(
    path.join(this_directory, 'requirements.txt'),
    session=PipSession())

requirements = [str(ir.req) for ir in parsed_requirements]

setup(
    name="voicen",
    version="1.0.3",
    author="Voicen LLC",
    author_email="info@voicen.com",
    description="The python SDK developed by Voicen allows customers to use speech-to-text service of Voicen.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/voicen-llc/voicen-python-sdk",
    packages=find_packages("src"),
		package_dir={'': 'src'},
		py_modules=[os.path.splitext(os.path.basename(path))[0]
                for path in glob('src/*.py')],
		include_package_data=True,
		install_requires=requirements,
		zip_safe=False,
		license='MIT license',
		keywords='voicen',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)



from enum import Enum

class SubscriptionStatus(Enum):
    ACTIVE = 1
    EXPIRED = 2
    SUSPENDED = 3
    CANCELLED = 4
    TERMINATED = 5

    def to_string(self):
        return self.name.lower()

    @classmethod
    def from_string(cls, status):
        return cls[status.upper()]
